package com.dsic.gj_erp.service.sy.impl;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.sy.DmDepart;
import com.dsic.gj_erp.bean.sy.DmDepartIn;
import com.dsic.gj_erp.mapper.DmDepartMapper;
import com.dsic.gj_erp.service.cache.RedisCacheKeyConfig;
import com.dsic.gj_erp.service.sy.DmDepartService;
import com.dsic.gj_erp.util.ConditionSelect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.apache.commons.lang3.StringUtils;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 部门表 服务实现类
 * </p>
 *
 * @author xhj
 * @since 2021-03-23
 */
@Service
public class DmDepartServiceImpl extends ServiceImpl<DmDepartMapper, DmDepart> implements DmDepartService {

    @Resource
    private DmDepartMapper dmDepartMapper;

    private HashMap<String, String> departBm = new HashMap<String, String>(){{
        put("DY", "03");
        put("GJ", "02");
        put("TX", "01");
        put("QC", "06");
    }};

    /**
     * 无参查询所有部门
     * @author 张恩铭
     * @return
     */
    @Override
    public List<DmDepart> getAll() {
        return this.dmDepartMapper.selectList(null);
    }

    /**
     * 根据部门号条件查询部门及其子部门
     * @author 张恩铭
     * @param dmDepart
     * @return
     */
    @Override
    public List<DmDepart> getByCondition(DmDepart dmDepart) {
        ConditionSelect<DmDepart> conditionSelect = new ConditionSelect<>(dmDepart, new QueryWrapper<>(), dmDepartMapper);
        return conditionSelect.getResultByLikeRight();
    }

    /**
     * 部门号模糊查询其他条件指定查询
     * @param dmDepart
     * @return
     */
    @Override
    public List<DmDepart> getByConditionDefaultDepartCode(DmDepartIn dmDepart) {
//        System.out.println(dmDepart);
//        VoToEntity<DmDepartIn, DmDepart> voToEntity = new VoToEntity<>(dmDepart, new DmDepart());
//        DmDepart eneity = voToEntity.getEneity();
//        boolean flag = false;
//        if(dmDepart.getDepartCode().length() >= 2){
//            for (String key : departBm.keySet()) {
//                if (dmDepart.getZxtbm().equals(key)){
//                    flag = true;
//                    break;
//                }
//            }
//            if (flag){
//                return getByCondition(eneity);
//            }else{
//                eneity.setDepartCode(departBm.get(dmDepart.getZxtbm()));
//                return getByCondition(eneity);
//            }
//        }else {
//            eneity.setDepartCode(departBm.get(dmDepart.getZxtbm()));
//            return getByCondition(eneity);
//        }
            QueryWrapper<DmDepart> qw =new QueryWrapper<>();
            qw.eq("parent_code",dmDepart.getParentCode());
          if(StringUtils.isNotBlank(dmDepart.getDepartCode())){
            qw.eq("depart_code",dmDepart.getDepartCode());
          }
        if(StringUtils.isNotBlank(dmDepart.getDepartCode())){
            qw.eq("depart_code",dmDepart.getDepartCode());
        }
        if(StringUtils.isNotBlank(dmDepart.getManuType())){
            qw.eq("manu_type",dmDepart.getManuType());
        }
        if(StringUtils.isNotBlank(dmDepart.getDepartType())){
            qw.eq("depart_type",dmDepart.getDepartType());
        }
        if(StringUtils.isNotBlank(dmDepart.getDepartAttr())){
            qw.eq("depart_attr",dmDepart.getDepartAttr());
        }
        qw.eq(StrUtil.isNotEmpty(dmDepart.getManuFlag()),"manu_flag",dmDepart.getManuFlag());
            return this.list(qw);
    }

    /**
     * 查询所有主部门
     * @author 张恩铭
     * @return
     */
    @Override
    public List<DmDepart> getAllParent() {
        List<DmDepart> result = new ArrayList<>();
        List<DmDepart> temp = this.getAll();
        for (DmDepart d : temp) {
            if(d.getDepartCode().length() == 2){
                result.add(d);
            }
        }
        return result;
    }

    /**
     * 查询所有子部门
     * @author 张恩铭
     * @return
     */
    @Override
    public List<DmDepart> getAllChildren() {
        List<DmDepart> result = new ArrayList<>();
        List<DmDepart> temp = this.getAll();
        for (DmDepart d : temp) {
            if(d.getDepartCode().length() > 2){
                result.add(d);
            }
        }
        return result;
    }

    /**
     * 添加部门
     * @author 张恩铭
     * @param dmDeparts
     * @return
     */
    @Override
    public ResultBean<Integer> addDepart(DmDepart[] dmDeparts) {
        int i = 0;
        for (DmDepart d : dmDeparts) {
            if(d.getDepartCode() == null || d.getDepartCode().equals("")){
                continue;
            }else{
                DmDepart temp = new DmDepart();
                temp.setDepartCode(d.getDepartCode());
                ConditionSelect<DmDepart> conditionSelect = new ConditionSelect<>(temp, new QueryWrapper<>(), this.dmDepartMapper);
                if(conditionSelect.getResult().size() > 0){
                    continue;
                }else{
                    if(this.dmDepartMapper.insert(d) == 1) i++;
                }
            }
        }
        ResultBean<Integer> resultBean = new ResultBean<>();
        resultBean.setData(i);
        return resultBean;
    }

    /**
     * 修改部门
     * @author 张恩铭
     * @param dmDeparts
     * @return
     */
    @Override
    public ResultBean<Integer> updateDepart(DmDepart[] dmDeparts) {
        int i = 0;
        for (DmDepart d : dmDeparts) {
            if(d.getDepartCode() == null || d.getDepartCode().equals("")){
                continue;
            }else{
                DmDepart temp = new DmDepart();
                temp.setDepartCode(d.getDepartCode());
                ConditionSelect<DmDepart> conditionSelect = new ConditionSelect<>(temp, new QueryWrapper<>(), this.dmDepartMapper);
                List<DmDepart> result = conditionSelect.getResult();
                if(result.size() > 0 && !result.get(0).getDepartCode().equals(d.getDepartCode())){
                    continue;
                }else{
                    if(this.dmDepartMapper.updateById(d) == 1) i++;
                }
            }
        }
        ResultBean<Integer> resultBean = new ResultBean<>();
        resultBean.setData(i);
        return resultBean;
    }

    /**
     * 删除部门
     * @author 张恩铭
     * @param dmDeparts
     * @return
     */
    @Override
    public ResultBean<Integer> delDepart(DmDepart[] dmDeparts) {
        int i = 0;
        for (DmDepart d : dmDeparts) {
            if(this.dmDepartMapper.deleteById(d) == 1) i++;
        }
        ResultBean<Integer> resultBean = new ResultBean<>();
        resultBean.setData(i);
        return resultBean;
    }
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<List<DmDepart>> selectDepartQuery( List<String> params) {
        ResultBean<List<DmDepart>> resultBean = new ResultBean<List<DmDepart>>();
        resultBean.setData(dmDepartMapper.selectDepartQuery(params));
        return resultBean;

    }

    @Resource
    RedisTemplate<String ,Object> redisTemplate;
    @Override
    public void refreshRedis() {
        List<DmDepart> list=this.list();
        list.forEach(l->redisTemplate.opsForHash().put(RedisCacheKeyConfig.DEPT_KEY,l.getDepartCode(),l));
    }
}
