package com.dsic.gj_erp.util;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.List;

/**
 * @version V1.0.0
 * @Title: ConditionSelect
 * @Package com.dsic.xzerp.bean
 * @Description: 条件查询
 * @author: 张恩铭
 * @date: 2021/3/16 11:28
 */
public class ConditionSelect<T> implements Serializable {

    private T entity;
    private QueryWrapper<T> queryWrapper;
    private BaseMapper<T> baseMapper;
    private String[] likeAttrs;

    public ConditionSelect(T data, QueryWrapper<T> queryWrapper, BaseMapper<T> baseMapper){
        this.entity = data;
        this.queryWrapper = queryWrapper;
        this.baseMapper = baseMapper;
    }

    public ConditionSelect(T data, QueryWrapper<T> queryWrapper, BaseMapper<T> baseMapper, String[] likeAttrs){
        this.entity = data;
        this.queryWrapper = queryWrapper;
        this.baseMapper = baseMapper;
        this.likeAttrs = likeAttrs;
    }

    /**
     * 无like不定条件查询
     * @return java.util.List
     */
    public List<T> getResult(){
        try{
            Field[] fields = entity.getClass().getDeclaredFields();

            for (Field f:fields) {
                if (f.getAnnotation(TableField.class)!=null&&!f.getAnnotation(TableField.class).exist()){
                    continue;
                }
                f.setAccessible(true);
                if (!f.getName().equals("serialVersionUID")){
                    if(f.get(entity) != null && f.get(entity) != "" && f.get(entity).toString().toLowerCase() != "null"){
                        char[] chars = f.getName().toCharArray();
                        String temp_s = "";
                        for (int i = 0; i < chars.length; i++){
                            if(chars[i] >= 'A' && chars[i] <= 'Z'){
                                String temp_ss = String.valueOf(chars[i]);
                                temp_s = temp_s + "_" + temp_ss.toLowerCase();
                            }else{
                                temp_s += chars[i];
                            }
                        }
                        queryWrapper.eq(temp_s, f.get(entity));
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        List<T> list = this.baseMapper.selectList(this.queryWrapper);
        return list;
    }

    /**
     * 只有like不定条件查询
     * @return java.util.List
     */
    public List<T> getResultByLikeRight(){
        try{
            Field[] fields = entity.getClass().getDeclaredFields();
            for (Field f:fields) {
                f.setAccessible(true);
                if (!f.getName().equals("serialVersionUID")){
                    if(f.get(entity) != null && f.get(entity) != "" && f.get(entity).toString().toLowerCase() != "null"){
                        char[] chars = f.getName().toCharArray();
                        String temp_s = "";
                        for (int i = 0; i < chars.length; i++){
                            if(chars[i] >= 'A' && chars[i] <= 'Z'){
                                String temp_ss = String.valueOf(chars[i]);
                                temp_s = temp_s + "_" + temp_ss.toLowerCase();
                            }else{
                                temp_s += chars[i];
                            }
                        }
                        queryWrapper.likeRight(temp_s, f.get(entity));
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        List<T> list = this.baseMapper.selectList(this.queryWrapper);
        return list;
    }

    /**
     * 特定like不定条件查询
     * @return java.util.List
     */
    public List<T> getResultBySomeLikeRight(){
        boolean flag = false;
        try {
            Field[] fields = entity.getClass().getDeclaredFields();
            for (Field f:fields) {
                f.setAccessible(true);
                if (!f.getName().equals("serialVersionUID")){
                    if(f.get(entity) != null && f.get(entity) != "" && f.get(entity).toString().toLowerCase() != "null"){
                        for (String like_string: likeAttrs) {
                            if(like_string.equals(f.getName())){
                                flag = true;
                            }
                            break;
                        }
                        char[] chars = f.getName().toCharArray();
                        String temp_s = "";
                        for (int i = 0; i < chars.length; i++){
                            if(chars[i] >= 'A' && chars[i] <= 'Z'){
                                String temp_ss = String.valueOf(chars[i]);
                                temp_s = temp_s + "_" + temp_ss.toLowerCase();
                            }else{
                                temp_s += chars[i];
                            }
                        }
                        if(!flag){
                            queryWrapper.eq(temp_s, f.get(entity));
                        }else{
                            queryWrapper.likeRight(temp_s, f.get(entity));
                        }
                    }
                }
                flag = false;
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        List<T> list = this.baseMapper.selectList(this.queryWrapper);
        return list;
    }

}
